@@ -7,6 +7,7 @@ setuptools entry points.
 
 from __future__ import absolute_import
 from __future__ import unicode_literals
+from __future__ import print_function
 
 # import udiskie.depend first - for side effects!
 from .depend import has_Notify, has_Gtk, _in_X
@@ -22,6 +23,7 @@ from gi.repository import GLib
 import udiskie
 import udiskie.config
 import udiskie.mount
+import udiskie.compat
 from .async_ import AsyncList, Coroutine, Return, RunForever
 from .common import extend, str2unicode
 from .locale import _
@@ -165,6 +167,7 @@ class _EntryPoint(object):
 
         :param list argv: command line parameters
         """
+        udiskie.compat.patch_print_unicode()
         # parse program options (retrieve log level and config file name):
         args = docopt(self.usage, version=self.name + ' ' + self.version)
         default_opts = self.option_defaults
@@ -326,6 +329,10 @@ class Daemon(_EntryPoint):
         -t, --tray                              Show tray icon
         -s, --smart-tray                        Auto hide tray icon
         -T, --no-tray                           Disable tray icon
+        -m MENU, --menu MENU                    Tray menu [smart/nested/flat]
+
+        --appindicator                          Use appindicator for status icon
+        --no-appindicator                       Don't use appindicator
 
         --password-cache MINUTES                Set password cache timeout
         --no-password-cache                     Disable password cache
@@ -344,6 +351,8 @@ class Daemon(_EntryPoint):
         'automount': True,
         'notify': True,
         'tray': False,
+        'menu': 'flat',
+        'appindicator': False,
         'file_manager': 'xdg-open',
         'password_prompt': 'builtin:gui',
         'password_cache': False,
@@ -356,6 +365,8 @@ class Daemon(_EntryPoint):
             '--tray': True,
             '--no-tray': False,
             '--smart-tray': 'auto'}),
+        'menu': Value('--menu'),
+        'appindicator': Switch('appindicator'),
         'file_manager': OptionalValue('--file-manager'),
         'password_prompt': OptionalValue('--password-prompt'),
         'password_cache': OptionalValue('--password-cache'),
@@ -434,20 +445,31 @@ class Daemon(_EntryPoint):
         # tray icon (optional):
         if options['tray']:
             import udiskie.tray
-            tray_classes = {True: udiskie.tray.TrayIcon,
-                            'auto': udiskie.tray.AutoTray}
-            if options['tray'] not in tray_classes:
+            if options['tray'] == 'auto':
+                smart = True
+            elif options['tray'] is True:
+                smart = False
+            else:
                 raise ValueError("Invalid tray: %s" % (options['tray'],))
             icons = udiskie.tray.Icons(config.icon_names)
             actions = udiskie.mount.DeviceActions(mounter)
-            menu_maker = udiskie.tray.SmartUdiskieMenu(
-                mounter,
-                icons,
-                actions,
-                quit_action=self.mainloop.quit)
-            TrayIcon = tray_classes[options['tray']]
-            statusicon = TrayIcon(menu_maker, icons)
-            tasks.append(statusicon.task)
+
+            menu_classes = {'smart': udiskie.tray.SmartUdiskieMenu,
+                            'nested': udiskie.tray.UdiskieMenu,
+                            'flat': udiskie.tray.FlatUdiskieMenu}
+            if options['menu'] not in menu_classes:
+                raise ValueError("Invalid menu: %s" % (options['tray'],))
+            Menu = menu_classes[options['menu']]
+            menu_maker = Menu(mounter, icons, actions,
+                              quit_action=self.mainloop.quit)
+            if options['appindicator']:
+                import udiskie.appindicator
+                TrayIcon = udiskie.appindicator.AppIndicatorIcon
+            else:
+                TrayIcon = udiskie.tray.TrayIcon
+            trayicon = TrayIcon(menu_maker, icons)
+            statusicon = udiskie.tray.UdiskieStatusIcon(trayicon, menu_maker, smart)
+            tasks.append(trayicon.task)
         else:
             statusicon = None
             tasks.append(RunForever)
@@ -503,7 +525,7 @@ class Mount(_EntryPoint):
     name = 'udiskie-mount'
 
     option_defaults = extend(_EntryPoint.option_defaults, {
-        'recursive': False,
+        'recursive': None,
         'options': None,
         '<device>': None,
         'password_prompt': 'builtin:tty',
@@ -589,7 +611,7 @@ class Umount(_EntryPoint):
     name = 'udiskie-umount'
 
     option_defaults = extend(_EntryPoint.option_defaults, {
-        'detach': False,
+        'detach': None,
         'eject': False,
         'force': False,
         'lock': True,
@@ -608,8 +630,12 @@ class Umount(_EntryPoint):
 
         """Implements _EntryPoint._init."""
 
+        config = self.config
         options = self.options
-        mounter = udiskie.mount.Mounter(self.udisks)
+
+        mounter = udiskie.mount.Mounter(
+            self.udisks,
+            ignore_device=config.ignore_device)
 
         strategy = dict(detach=options['detach'],
                         eject=options['eject'],
@@ -621,3 +647,91 @@ class Umount(_EntryPoint):
         else:
             tasks = [mounter.remove_all(**strategy)]
         return AsyncList(tasks)
+
+
+def _parse_filter(spec):
+    try:
+        key, val = spec.split('=', 1)
+    except ValueError:
+        if spec.startswith('!'):
+            val = False
+            key = spec[1:]
+        else:
+            val = True
+            key = spec
+    return key, val
+
+
+class Info(_EntryPoint):
+
+    """
+    udiskie-info: get information about handleable devices.
+
+    Usage:
+        udiskie-info [options] [-o OUTPUT] [-f FILTER]... (-a | DEVICE...)
+        udiskie-info (--help | --version)
+
+    General options:
+        -c FILE, --config=FILE      Set config file
+        -C, --no-config             Don't use config file
+
+        -v, --verbose               Increase verbosity (DEBUG)
+        -q, --quiet                 Decrease verbosity
+
+        -0, --udisks-auto           Auto discover UDisks version
+        -1, --use-udisks1           Use UDisks1 as backend
+        -2, --use-udisks2           Use UDisks2 as backend
+
+        -h, --help                  Show this help
+        -V, --version               Show version information
+
+    Unmount options:
+        -a, --all                   List all handleable devices
+
+        -o COL, --output COL        Specify which output column to print
+                                    [default: device_presentation].
+
+        -f FILT, --filter FILT      Print only devices that match the given
+                                    filter.
+    """
+
+    name = 'udiskie-info'
+
+    option_defaults = extend(_EntryPoint.option_defaults, {
+        'output': '',
+        'filter': '',
+        '<device>': None,
+    })
+
+    option_rules = extend(_EntryPoint.option_rules, {
+        'output': Value('--output'),
+        'filter': Value('--filter'),
+        '<device>': Value('DEVICE'),
+    })
+
+    def _init(self):
+
+        """Implements _EntryPoint._init."""
+
+        config = self.config
+        options = self.options
+
+        mounter = udiskie.mount.Mounter(
+            self.udisks,
+            ignore_device=config.ignore_device)
+
+        if options['<device>']:
+            devices = [self.udisks.find(path) for path in options['<device>']]
+        else:
+            devices = mounter.get_all_handleable()
+
+        output = options['output']
+
+        filters = [_parse_filter(spec) for spec in options['filter']]
+        matcher = udiskie.config.DeviceFilter(dict(filters), None)
+
+        for device in devices:
+            if matcher.match(device):
+                print(getattr(device, output))
+
+        return AsyncList([])
