@@ -153,6 +153,93 @@ class AttrDictView(object):
             raise AttributeError
 
 
+@fix_str_conversions
+class BaseDevice(object):
+
+    def __str__(self):
+        """Show as object_path."""
+        return self.object_path
+
+    def __eq__(self, other):
+        """Comparison by object_path."""
+        return self.object_path == str(other)
+
+    def __ne__(self, other):
+        """Comparison by object_path."""
+        return not (self == other)
+
+    def is_file(self, path):
+        """Comparison by mount and device file path."""
+        return (samefile(path, self.device_file) or
+                samefile(path, self.loop_file) or
+                any(samefile(path, mp) for mp in self.mount_paths) or
+                sameuuid(path, self.id_uuid) or
+                sameuuid(path, self.partition_uuid))
+
+    # ----------------------------------------
+    # derived properties
+    # ----------------------------------------
+
+    @property
+    def mount_path(self):
+        """Return any mount path."""
+        try:
+            return self.mount_paths[0]
+        except IndexError:
+            return ''
+
+    @property
+    def in_use(self):
+        """Check whether this device is in use, i.e. mounted or unlocked."""
+        if self.is_mounted or self.is_unlocked:
+            return True
+        if self.is_partition_table:
+            for device in self._daemon:
+                if device.partition_slave == self and device.in_use:
+                    return True
+        return False
+
+    @property
+    def ui_id_label(self):
+        """Label of the unlocked partition or the device itself."""
+        return (self.luks_cleartext_holder or self).id_label
+
+    @property
+    def ui_id_uuid(self):
+        """UUID of the unlocked partition or the device itself."""
+        return (self.luks_cleartext_holder or self).id_uuid
+
+    @property
+    def ui_device_presentation(self):
+        """Path of the crypto backing device or the device itself."""
+        return (self.luks_cleartext_slave or self).device_presentation
+
+    @property
+    def ui_label(self):
+        """UI string identifying the partition if possible."""
+        return ': '.join(filter(None, [
+            self.ui_device_presentation,
+            self.ui_id_label or self.ui_id_uuid or self.drive_label
+        ]))
+
+    @property
+    def ui_device_label(self):
+        """UI string identifying the device (drive) if toplevel."""
+        return ': '.join(filter(None, [
+            self.ui_device_presentation,
+            self.loop_file or
+            self.drive_label or self.ui_id_label or self.ui_id_uuid
+        ]))
+
+    @property
+    def drive_label(self):
+        """Return drive label."""
+        return ' '.join(filter(None, [
+            self.drive_vendor,
+            self.drive_model,
+        ]))
+
+
 @fix_str_conversions
 class NullDevice(object):
 
@@ -274,11 +361,29 @@ class NullDevice(object):
 
     # Loop
     loop_file = ''
+    setup_by_uid = -1
+    autoclear = None
+
+    def delete(self, auth_no_user_interaction=None):
+        raise RuntimeError("Cannot call methods on invalid device!")
+
+    def set_autoclear(self, value, auth_no_user_interaction=None):
+        raise RuntimeError("Cannot call methods on invalid device!")
+
+    loop_support = False
 
     # derived properties
+    mount_path = ''
     in_use = False
-    parent_object_path = '/'
+    ui_id_label = ''
+    ui_id_uuid = ''
+    ui_device_presentation = ''
     ui_label = '(invalid device)'
+    ui_device_label = ''
+    drive_label = ''
+    parent_object_path = '/'
+    can_add = False
+    can_remove = False
 
 
 # ----------------------------------------
