@@ -9,6 +9,7 @@ import logging
 
 from gi.repository import GLib
 
+from .common import exc_message
 from .mount import DeviceActions
 from .locale import _
 
@@ -60,6 +61,8 @@ class Notify(object):
 
         :param device: device object
         """
+        if not self._mounter.is_handleable(device):
+            return
         browse_action = ('browse', _('Browse directory'),
                          self._mounter.browse, device)
         self._show_notification(
@@ -75,6 +78,8 @@ class Notify(object):
 
         :param device: device object
         """
+        if not self._mounter.is_handleable(device):
+            return
         self._show_notification(
             'device_unmounted',
             _('Device unmounted'),
@@ -87,6 +92,8 @@ class Notify(object):
 
         :param device: device object
         """
+        if not self._mounter.is_handleable(device):
+            return
         self._show_notification(
             'device_locked',
             _('Device locked'),
@@ -99,6 +106,8 @@ class Notify(object):
 
         :param device: device object
         """
+        if not self._mounter.is_handleable(device):
+            return
         self._show_notification(
             'device_unlocked',
             _('Device unlocked'),
@@ -111,6 +120,8 @@ class Notify(object):
 
         :param device: device object
         """
+        if not self._mounter.is_handleable(device):
+            return
         if self._has_actions('device_added'):
             # wait for partitions etc to be reported to udiskie, otherwise we
             # can't discover the actions
@@ -129,8 +140,8 @@ class Notify(object):
                 flat_actions = self._flatten_node(node_tree)
                 actions = [
                     (action.method,
-                    action.label.format(action.device.ui_label),
-                    action.action)
+                     action.label.format(action.device.ui_label),
+                     action.action)
                     for action in flat_actions
                 ]
             else:
@@ -155,6 +166,8 @@ class Notify(object):
 
         :param device: device object
         """
+        if not self._mounter.is_handleable(device):
+            return
         device_file = device.device_presentation
         if (device.is_drive or device.is_toplevel) and device_file:
             self._show_notification(
@@ -169,6 +182,8 @@ class Notify(object):
 
         :param device: device object
         """
+        if not self._mounter.is_handleable(device):
+            return
         device_file = device.device_presentation or device.object_path
         if message:
             text = _('failed to {0} {1}:\n{2}', action, device_file, message)
@@ -203,7 +218,7 @@ class Notify(object):
         if timeout != -1:
             notification.set_timeout(int(timeout * 1000))
         for action in actions:
-            if self._action_enabled(event, action[0]):
+            if action and self._action_enabled(event, action[0]):
                 self._add_action(notification, *action)
         try:
             notification.show()
