@@ -23,7 +23,8 @@ import os.path
 from gi.repository import GLib
 
 from .async_ import AsyncList, Coroutine, Return
-from .common import Emitter, samefile, sameuuid, AttrDictView, wraps, NullDevice
+from .common import (Emitter, samefile, sameuuid, AttrDictView, wraps,
+                     NullDevice, BaseDevice)
 from .compat import fix_str_conversions
 from .dbus import connect_service, MethodsProxy
 from .locale import _
@@ -39,8 +40,7 @@ def filter_opt(opt):
     return [k for k, v in opt.items() if v]
 
 
-@fix_str_conversions
-class Device(object):
+class Device(BaseDevice):
 
     """Helper base class for devices."""
 
@@ -57,26 +57,6 @@ class Device(object):
         self._P = property_proxy
         self._M = method_proxy
 
-    def __str__(self):
-        """Display as object path."""
-        return self.object_path
-
-    def __eq__(self, other):
-        """Comparison by object path."""
-        return self.object_path == str(other)
-
-    def __ne__(self, other):
-        """Comparison by object path."""
-        return not (self == other)
-
-    def is_file(self, path):
-        """Comparison by mount and device file path."""
-        return (samefile(path, self.device_file) or
-                samefile(path, self.loop_file) or
-                any(samefile(path, mp) for mp in self.mount_paths) or
-                sameuuid(path, self.id_uuid) or
-                sameuuid(path, self.partition_uuid))
-
     # availability of interfaces
     @property
     def is_drive(self):
@@ -139,6 +119,16 @@ class Device(object):
         """Check if there is media available in the drive."""
         return self._P.DeviceIsMediaAvailable
 
+    @property
+    def drive_vendor(self):
+        """Return drive vendor."""
+        return self._P.DriveVendor
+
+    @property
+    def drive_model(self):
+        """Return drive model."""
+        return self._P.DriveModel
+
     # Drive methods
     def eject(self, unmount=False):
         """Eject media from the device."""
@@ -373,21 +363,27 @@ class Device(object):
         """Get the file backing the loop device."""
         return self._P.LinuxLoopFilename
 
+    @property
+    def setup_by_uid(self):
+        """[NOT IN UDISKS1] Get the ID of the user who set up the loop device."""
+        return None
+
+    @property
+    def autoclear(self):
+        """[NOT IN UDISKS1] If True the loop device will be deleted after unmounting."""
+        return None
+
+    def delete(self, *args, **kwargs):
+        raise NotImplementedError("No support for manipulating loop devices in UDisks1!")
+
+    set_autoclear = delete
+
+    loop_support = False
+
     # ----------------------------------------
     # derived properties
     # ----------------------------------------
 
-    @property
-    def in_use(self):
-        """Check whether this device is in use, i.e. mounted or unlocked."""
-        if self.is_mounted or self.is_unlocked:
-            return True
-        if self.is_partition_table:
-            for device in self._daemon:
-                if device.partition_slave == self and device.in_use:
-                    return True
-        return False
-
     @property
     def parent_object_path(self):
         if self.is_partition:
@@ -397,10 +393,6 @@ class Device(object):
         else:
             return '/'
 
-    @property
-    def ui_label(self):
-        return self.id_label or self.id_uuid or self.device_presentation
-
 
 def _keep_async_event_order(func):
     """
@@ -529,11 +521,17 @@ class Daemon(Emitter):
     @classmethod
     @Coroutine.from_generator_function
     def create(cls):
-        proxy = yield connect_service(cls)
+        service = (cls.BusName, cls.ObjectPath, cls.Interface)
+        proxy = yield connect_service(*service)
         udisks = cls(proxy)
         yield udisks._sync()
         yield Return(udisks)
 
+    def loop_setup(self, fd, options):
+        raise NotImplementedError("UDisks1 does not support LoopSetup!")
+
+    loop_support = False
+
     # Sniffer overrides
     def paths(self):
         """Iterate over all valid cached devices."""
