@@ -18,9 +18,9 @@ import logging
 
 from gi.repository import GLib
 
-from .common import Emitter, samefile, sameuuid, AttrDictView, decode_ay
-from .compat import fix_str_conversions
-from .dbus import connect_service, MethodsProxy
+from .common import (Emitter, samefile, sameuuid, AttrDictView, decode_ay,
+                     BaseDevice)
+from .dbus import connect_service, MethodsProxy, DBusCallWithFdList
 from .locale import _
 from .async_ import Coroutine, Return
 
@@ -117,8 +117,7 @@ class PropertiesNotAvailable(object):
 # Device wrapper
 # ----------------------------------------
 
-@fix_str_conversions
-class Device(object):
+class Device(BaseDevice):
 
     """
     Proxy class for UDisks2 devices.
@@ -135,26 +134,6 @@ class Device(object):
         self._P = property_hub
         self._M = method_hub
 
-    def __str__(self):
-        """Show as object_path."""
-        return self.object_path
-
-    def __eq__(self, other):
-        """Comparison by object_path."""
-        return self.object_path == str(other)
-
-    def __ne__(self, other):
-        """Comparison by object_path."""
-        return not (self == other)
-
-    def is_file(self, path):
-        """Comparison by mount and device file path."""
-        return (samefile(path, self.device_file) or
-                samefile(path, self.loop_file) or
-                any(samefile(path, mp) for mp in self.mount_paths) or
-                sameuuid(path, self.id_uuid) or
-                sameuuid(path, self.partition_uuid))
-
     # availability of interfaces
     @property
     def is_drive(self):
@@ -229,6 +208,16 @@ class Device(object):
         """Check if there is media available in the drive."""
         return bool(self._assocdrive._P.Drive.MediaAvailable)
 
+    @property
+    def drive_vendor(self):
+        """Return drive vendor."""
+        return self._assocdrive._P.Drive.Vendor
+
+    @property
+    def drive_model(self):
+        """Return drive model."""
+        return self._assocdrive._P.Drive.Model
+
     # Drive methods
     def eject(self, auth_no_user_interaction=None):
         """Eject media from the device."""
@@ -505,31 +494,47 @@ class Device(object):
         """Get the file backing the loop device."""
         return decode_ay(self._P.Loop.BackingFile)
 
+    @property
+    def setup_by_uid(self):
+        """Get the ID of the user who set up the loop device."""
+        return self._P.Loop.SetupByUID
+
+    @property
+    def autoclear(self):
+        """If True the loop device will be deleted after unmounting."""
+        return self._P.Loop.Autoclear
+
+    def delete(self, auth_no_user_interaction=None):
+        """Delete loop partition."""
+        return self._M.Loop.Delete(
+            '(a{sv})',
+            filter_opt({
+                'auth.no_user_interaction': ('b', auth_no_user_interaction),
+            })
+        )
+
+    def set_autoclear(self, value, auth_no_user_interaction=None):
+        """Set autoclear flag for loop partition."""
+        return self._M.Loop.SetAutoclear(
+            '(ba{sv})',
+            value,
+            filter_opt({
+                'auth.no_user_interaction': ('b', auth_no_user_interaction),
+            })
+        )
+
+    loop_support = True
+
     # ----------------------------------------
     # derived properties
     # ----------------------------------------
 
-    @property
-    def in_use(self):
-        """Check whether this device is in use, i.e. mounted or unlocked."""
-        if self.is_mounted or self.is_unlocked:
-            return True
-        if self.is_partition_table:
-            for device in self._daemon:
-                if device.partition_slave == self and device.in_use:
-                    return True
-        return False
-
     @property
     def parent_object_path(self):
         return (self._P.Partition.Table
                 or self._P.Block.CryptoBackingDevice
                 or '/')
 
-    @property
-    def ui_label(self):
-        return self.id_label or self.id_uuid or self.device_presentation
-
 
 # ----------------------------------------
 # UDisks2 service wrapper
@@ -622,11 +627,34 @@ class Daemon(Emitter):
     @classmethod
     @Coroutine.from_generator_function
     def create(cls):
-        proxy = yield connect_service(cls)
+        service = (cls.BusName, cls.ObjectPath, cls.Interface)
+        proxy = yield connect_service(*service)
         daemon = cls(proxy)
         yield daemon._sync()
         yield Return(daemon)
 
+    @Coroutine.from_generator_function
+    def loop_setup(self, fd, options):
+        service = (self.BusName,
+                   '/org/freedesktop/UDisks2/Manager',
+                   Interface['Manager'])
+        manager = yield connect_service(*service)
+        object_path = yield DBusCallWithFdList(
+            manager._proxy, 'LoopSetup', '(ha{sv})',
+            (0, filter_opt({
+                'auth.no_user_interaction': ('b', options.get('auth.no_user_interaction')),
+                'offset': ('t', options.get('offset')),
+                'size': ('t', options.get('size')),
+                'read-only': ('b', options.get('read-only')),
+                'no-part-scan': ('b', options.get('no-part-scan')),
+            })),
+            [fd],
+        )
+        yield self._sync()
+        yield Return(self[object_path])
+
+    loop_support = True
+
     # UDisks2 interface
     def paths(self):
         return self._objects.keys()
